import java.io.BufferedReader;
import java.io.FileReader;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.StringTokenizer;
import java.util.ArrayList;

/**
 * Handels the cration of a DataSet.
 */
public class DataSetProducer {

    private static final int INITAL_DATASET_SIZE = 4000;

    private BufferedReader reader;

    /**
     * Initializes a DataSetProducer on the specified files.
     * @param fileName name of file to use as source data for the DataSet
     * @throws FileNotFoundException If the specified file does not exist.
     */
    public DataSetProducer(String fileName) throws FileNotFoundException {
        reader = new BufferedReader(new FileReader(fileName));
    }

    /**
     * Reads and parses the dataset from the source file
     * @return DataSet containing all the TimeSeries objects in the file
     * @throws IOException If the parsing was unsuccessful
     */
    public DataSet produceDataSet() throws IOException {
        //todo: Could add more robust parsing, but not really needed, assuming input is "correct"
        ArrayList<TimeSeries> dataSet = new ArrayList<TimeSeries>(INITAL_DATASET_SIZE);
        StringTokenizer tokenizer;
        int length;
        double[] timeSeries;

        //reading and creating the first TimeSeries outside the main loop to determine it's length, with wich all other TimeSeries will be compared for sanity reasons.
        tokenizer = new StringTokenizer(reader.readLine());
        timeSeries = new double[tokenizer.countTokens()];
        length = timeSeries.length;
        for (int i = 0; i < timeSeries.length; i++) {
            timeSeries[i] = Double.parseDouble(tokenizer.nextToken());
        }
        dataSet.add(new TimeSeries(timeSeries));

        //main loop
        String s = reader.readLine();
        while (s != null) {
            tokenizer = new StringTokenizer(s);
            if (tokenizer.countTokens() != length) {
                throw new IOException("Timeseries not of equal length");
            }
            timeSeries = new double[length];
            for (int i = 0; i < timeSeries.length; i++) {
                timeSeries[i] = Double.parseDouble(tokenizer.nextToken());
            }
            dataSet.add(new TimeSeries(timeSeries));

            s = reader.readLine();
        }

        return new DataSet(arrayListToTimeSeriesArray(dataSet));

    }

    /**
     * Internal metehod for some arraylist to primitive conversion
      * @param a arraylist to convert
     * @return resulting primitive
     */
    private static TimeSeries[] arrayListToTimeSeriesArray(ArrayList<TimeSeries> a) {
        TimeSeries[] ts = new TimeSeries[a.size()];
        for (int i = 0 ; i < ts.length ; i++) {
            ts[i] = a.get(i);
        }
        return ts;
    }

}
