import java.util.ArrayList;

/**
 * Created by IntelliJ IDEA.
 * User: e3r
 * Date: 31.okt.2007
 * Time: 19:50:31
 * To change this template use File | Settings | File Templates.
 */
public class LAESA {

    private static final String NUMBER_OF_PIVOTS_CONFIG_STRING = "LAESA-NumberOfPivots";
    private static final String NUMBER_OF_TEST_PAIRS_CONFIG_STRING = "LAESA-NumberOfTestPairs";
    private static final String NUMBER_OF_SAMPLE_PIVOTS_CONFIG_STRING = "LAESA-NumberOfSamplePivots";

    private DataSet dataSet;
    private DistanceMeasure distanceMeasure;
    private TimeSeries[] pivots;
    private double[][] distanceMap;

    private int distanceCalculationCounter;

    public void construct(DataSet ds, DistanceMeasure dm) {
        dataSet = ds;
        distanceMeasure = dm;
        distanceCalculationCounter = 0;
        pivots = new TimeSeries[Globals.getInt(NUMBER_OF_PIVOTS_CONFIG_STRING)];
        distanceMap = new double[pivots.length][dataSet.size()];
        double[] accu = new double[dataSet.size()]; //accumulator setting it to 0
        for (int i = 0; i < accu.length; i++) {
            accu[i] = 0;
        }
        int min;
        double minValue;
        int[] used = new int[accu.length];
        for (int i = 0; i < used.length; i++) {
            used[i] = -1;
        }
        int usedCounter = 0;
        //pick random pivot
        pivots[0] = dataSet.get((int) (dataSet.size() * Math.random()));
        for (int p = 0; p < distanceMap.length; p++) {
            for (int t = 0; t < distanceMap[0].length; t++) {
                distanceMap[p][t] = distanceMeasure.calculateDistance(pivots[p], dataSet.get(t));
                accu[t] += distanceMap[p][t];
            }

            min = -1;
            minValue = Double.MAX_VALUE;
            for (int i = 0; i < accu.length; i++) {
                if (accu[i] < minValue) {
                    boolean jus = false;
                    for (int x = 0; x < usedCounter; x++) {
                        if (used[x] == i) {
                            jus = true;
                            break;
                        }
                    }
                    if (!jus) {
                        min = i;
                        minValue = accu[i];
                    }
                }
            }
            if (min == -1) {
                System.err.println("ERROR: Number of Pivots greater then Number of Time Series (probably)");
                System.exit(-1);
            }
            used[usedCounter++] = min;
            if (p < distanceMap.length - 1) { //quick hax fix!
                pivots[p + 1] = dataSet.get(min);
            }
        }
    }

    private static TimeSeries[] makePivots(int numberOfPivots, DataSet ds, DistanceMeasure dm) {
        //create test pairs
        int numberOfTestPairs = Globals.getInt(NUMBER_OF_TEST_PAIRS_CONFIG_STRING);
        class Pair {
            TimeSeries a;
            TimeSeries b;
        }
        Pair[] pairs = new Pair[numberOfTestPairs];
        for (int i = 0; i < numberOfTestPairs; i++) {
            pairs[i].a = ds.get((int) (Math.random() * ds.size()));
            do {
                pairs[i].b = ds.get((int) (Math.random() * ds.size()));
            }
            while (pairs[i].a == pairs[i].b);
        }
        //init some vars
        double[] dMax = new double[numberOfPivots];
        int numberOfSamples = Globals.getInt(NUMBER_OF_SAMPLE_PIVOTS_CONFIG_STRING);
        double[] mys;
        double maxD, tmp;

        for (int i = 0; i < numberOfPivots; i++) {
            ArrayList<TimeSeries> samples = new ArrayList<TimeSeries>(numberOfSamples);
            while (samples.size() < numberOfSamples) {
                TimeSeries samp = ds.get((int) (Math.random() * ds.size()));
                if (!samples.contains(samp)) {
                    samples.add(samp);
                }
            }
            double[] maxDs = new double[samples.size()];
            for (int s = 0; s < samples.size(); s++) {
                maxD = 0;
                for (int p = 0; p < pairs.length; p++) {
                    tmp = Math.abs(dm.calculateDistance(samples.get(s), pairs[p].a) - dm.calculateDistance(samples.get(s), pairs[p].b));
                    maxD = Math.max(maxD, tmp);
                }
                maxDs[s] = maxD;
            }

        }

        //pick first pivot sample set

        //sample it
        //pick nest pivot sample set
        //etc
        return null;
    }

    public TimeSeries[] radiusQueryRough(TimeSeries query, double r) {
        ArrayList<TimeSeries> candidats = new ArrayList<TimeSeries>(2000);
        double distance;
        int counter = pivots.length;
        for (int p = 0; p < pivots.length; p++) {
            distance = distanceMeasure.calculateDistance(pivots[p], query);
            distanceCalculationCounter++;
            for (int i = 0; i < distanceMap[0].length; i++) {
                if (distanceMap[p][i] < distance + r && distanceMap[p][i] > distance - r) {
                    if (!candidats.contains(dataSet.get(i)))
                    {          //todo: quick fix, could do better, but not really noticable on runtime
                        candidats.add(dataSet.get(i));
                    }
                }
            }
        }

        TimeSeries[] result = new TimeSeries[candidats.size()];
        for (int i = 0; i < result.length; i++) {
            result[i] = candidats.get(i);
        }
        return result;
    }

    public void resetDistanceCalculationCounter() {
        distanceCalculationCounter = 0;
    }

    public int getDistanceCalculationCounter() {
        return distanceCalculationCounter;
    }
}
