import java.io.IOException;
import java.io.FileNotFoundException;
import java.io.BufferedWriter;
import java.io.FileWriter;
import java.util.ArrayList;

/**
 * Created by IntelliJ IDEA.
 * User: e3r
 * Date: 21.nov.2007
 * Time: 19:08:52
 * To change this template use File | Settings | File Templates.
 */
public class MITS {

    private static class LogWriter {
        private BufferedWriter bw;

        LogWriter(String fileName) throws IOException {
            bw = new BufferedWriter(new FileWriter(fileName));
        }

        void write(String s) {
            try {
                bw.write(s);
            } catch (IOException e) {
                System.err.println("ERROR: Writing to Log File failed!");
                System.exit(-1);
            }
        }

        void newline() {
            try {
                bw.newLine();
            } catch (IOException e) {
                System.err.println("ERROR: Writing to Log File failed!");
                System.exit(-1);
            }
        }

        void writeln(String s) {
            write(s);
            newline();
        }

        void close() {
            try {
                bw.close();
            } catch (IOException e) {
                System.err.println("ERROR: failed to properly close Log File");
                System.exit(-1);
            }
        }
    }

    public static void main(String[] args) {

        if (args.length < 4) {
            System.err.println("Invalid number of arguments:");
            System.err.println("config-file data-set-file query-file log-file");
            System.exit(-1);
        }

        LogWriter logWriter = null;
        try {
            logWriter = new LogWriter(args[3]);
        } catch (IOException e) {
            System.err.println("ERROR: Faild to initialize Log File");
            System.exit(-1);
        }

        logWriter.writeln("Reading configuration file " + args[0] + ".");
        try {
            new Globals(args[0]);
        } catch (IOException e) {
            logWriter.writeln("ERROR: Configuration file not found!");
            logWriter.writeln(e.getMessage());
            logWriter.close();
            System.err.println("ERROR: Config file not found");
            System.exit(-1);
        }
        logWriter.writeln("Configuration file read.");

        logWriter.writeln("Reading Data Set file " + args[1] + ".");
        DataSetProducer dp = null;
        try {
            dp = new DataSetProducer(args[1]);
        } catch (FileNotFoundException e) {
            logWriter.writeln("ERROR: Data Set file not found!");
            logWriter.writeln(e.getMessage());
            logWriter.close();
            System.err.println("ERROR: Dataset file not found");
            System.exit(-1);
        }

        DataSet ds = null;
        try {
            ds = dp.produceDataSet();
        } catch (IOException e) {
            logWriter.writeln("ERROR: Failed to read Data Set File");
            logWriter.writeln(e.getMessage());
            logWriter.close();
            System.err.println("ERROR: Failed to read Data Set");
            System.exit(-1);
        }
        logWriter.writeln("Data Set read.");

        logWriter.writeln("Reading Query File " + args[2] + ".");
        try {
            dp = new DataSetProducer(args[2]);
        } catch (FileNotFoundException e) {
            logWriter.writeln("ERROR: Query File not found");
            logWriter.writeln(e.getMessage());
            logWriter.close();
            System.err.println("ERROR: Query File not found");
            System.exit(-1);
        }
        DataSet query = null;
        try {
            query = dp.produceDataSet();
        } catch (IOException e) {
            logWriter.writeln("ERROR: Failed to read Query File");
            logWriter.writeln(e.getMessage());
            logWriter.close();
            System.err.println("ERROR: Failed to read Query Set");
            System.exit(-1);
        }
        logWriter.writeln("Query File Read.");

        logWriter.writeln("Initializing Distance Measure.");
        DistanceMeasure dm = new EuclideanDistanceMeasure();

        logWriter.writeln("Constructing LAESA index...");
        double timer = System.currentTimeMillis();
        LAESA laesa = new LAESA();
        laesa.construct(ds, dm);
        logWriter.writeln("LAESA index constructed in " + (System.currentTimeMillis() - timer) + " ms.");

        logWriter.writeln("Constructing LC index...");
        timer = System.currentTimeMillis();
        LC lc = new LC();
        lc.construct(ds, dm);
        logWriter.writeln("LC index constructed in " + (System.currentTimeMillis() - timer) + " ms.");

        logWriter.writeln("Generating PCA signatures...");
        timer = System.currentTimeMillis();
        PCA pca = new PCA();
        pca.construct(ds, dm);
        logWriter.writeln("PCA signatures generated in " + (System.currentTimeMillis() - timer) + " ms.");

        logWriter.writeln("Setting up Brute Force method...");
        timer = System.currentTimeMillis();
        BruteForce bruteForce = new BruteForce();
        bruteForce.construct(ds, dm);
        logWriter.writeln("Brute Force set up in " + (System.currentTimeMillis() - timer) + " ms.");

        System.currentTimeMillis();//Debug Breakpoint

        ArrayList<TimeSeries[]> resultsLAESA = new ArrayList<TimeSeries[]>(query.size());
        ArrayList<TimeSeries[]> resultsLC = new ArrayList<TimeSeries[]>(query.size());
        ArrayList<TimeSeries[]> resultsPCA = new ArrayList<TimeSeries[]>(query.size());
        ArrayList<TimeSeries[]> resultsBF = new ArrayList<TimeSeries[]>(query.size());
        int[] distanceCalculationsLAESA = new int[query.size()];
        int[] distanceCalculationsLC = new int[query.size()];
        int[] distanceCalculationsPCA = new int[query.size()];
        int[] distanceCalculationsBF = new int[query.size()];
        double searchRadious = Globals.getDouble("MITS-SearchRadius");
        logWriter.writeln("Starting Queries:");
        for (int i = 0; i < query.size(); i++) {
            logWriter.writeln("Query " + i + ":");

            logWriter.writeln("LAESA...");
            timer = System.currentTimeMillis();
            laesa.resetDistanceCalculationCounter();
            resultsLAESA.add(laesa.radiusQueryRough(query.get(i), searchRadious));
            distanceCalculationsLAESA[i] = laesa.getDistanceCalculationCounter();
            logWriter.writeln("done in " + (System.currentTimeMillis() - timer) + " ms");

            logWriter.writeln("LC...");
            timer = System.currentTimeMillis();
            lc.resetDistanceCalculationCounter();
            resultsLC.add(lc.radiusQueryRough(query.get(i), searchRadious));
            distanceCalculationsLC[i] = lc.getDistanceCalculationCounter();
            logWriter.writeln("done in " + (System.currentTimeMillis() - timer) + " ms");

            logWriter.writeln("PCA...");
            timer = System.currentTimeMillis();
            pca.resetDistanceCalculationCounter();
            resultsPCA.add(pca.radiusQueryRough(query.get(i), searchRadious));
            distanceCalculationsPCA[i] = pca.getDistanceCalculationCounter();
            logWriter.writeln("done in " + (System.currentTimeMillis() - timer) + " ms");

            logWriter.writeln("Brute Force...");
            timer = System.currentTimeMillis();
            bruteForce.resetDistanceCalculationCounter();
            resultsBF.add(bruteForce.radiusQueryRough(query.get(i), searchRadious));
            distanceCalculationsBF[i] = bruteForce.getDistanceCalculationCounter();
            logWriter.writeln("done in " + (System.currentTimeMillis() - timer) + " ms");
        }

        System.currentTimeMillis();//Debug Breakpoint

        logWriter.newline();
        logWriter.writeln("RESULTS");
        logWriter.writeln("Data set size: "+ds.size());

        for (int i = 0; i < query.size(); i++) {
            logWriter.newline();
            logWriter.writeln("QUERY NUMBER: " + i);
            int correctTotal = resultsBF.get(i).length;
            logWriter.writeln("Number of Correct results: " + correctTotal);

            logWriter.newline();
            logWriter.writeln("LAESA");
            int totalFound = resultsLAESA.get(i).length;
            int correctFound = compareResults(resultsLAESA.get(i), resultsBF.get(i));
            logWriter.writeln("Total found " + totalFound);
            logWriter.writeln("Correct found: " + correctFound);
            logWriter.writeln("Correct missed: " + (correctTotal - correctFound));
            logWriter.writeln("False found " + (totalFound - correctFound));
            logWriter.writeln("Search time Distance Calculations: "+distanceCalculationsLAESA[i]);
            logWriter.writeln("Total Distance Calculations: "+ (distanceCalculationsLAESA[i]+ totalFound));

            logWriter.newline();
            logWriter.writeln("LC");
            totalFound = resultsLC.get(i).length;
            correctFound = compareResults(resultsLC.get(i), resultsBF.get(i));
            logWriter.writeln("Total found " + totalFound);
            logWriter.writeln("Correct found: " + correctFound);
            logWriter.writeln("Correct missed: " + (correctTotal - correctFound));
            logWriter.writeln("False found " + (totalFound - correctFound));
            logWriter.writeln("Search time Distance Calculations: "+distanceCalculationsLC[i]);
            logWriter.writeln("Total Distance Calculations: "+ (distanceCalculationsLC[i]+ totalFound));

            logWriter.newline();
            logWriter.writeln("PCA");
            totalFound = resultsPCA.get(i).length;
            correctFound = compareResults(resultsPCA.get(i), resultsBF.get(i));
            logWriter.writeln("Total found " + totalFound);
            logWriter.writeln("Correct found: " + correctFound);
            logWriter.writeln("Correct missed: " + (correctTotal - correctFound));
            logWriter.writeln("False found " + (totalFound - correctFound));
            logWriter.writeln("Search time Distance Calculations: "+distanceCalculationsPCA[i]);
            logWriter.writeln("Total Distance Calculations: "+ (distanceCalculationsPCA[i]+ totalFound));

            logWriter.newline();
            logWriter.writeln("BruteForce");
            totalFound = resultsBF.get(i).length;
            correctFound = compareResults(resultsBF.get(i), resultsBF.get(i));
            logWriter.writeln("Total found " + totalFound);
            logWriter.writeln("Correct found: " + correctFound);
            logWriter.writeln("Correct missed: " + (correctTotal - correctFound));
            logWriter.writeln("False found " + (totalFound - correctFound));
            logWriter.writeln("Search time Distance Calculations: "+distanceCalculationsBF[i]);
            logWriter.writeln("Total Distance Calculations: "+ distanceCalculationsBF[i]);

        }

        logWriter.close();
    }

    private static int compareResults(TimeSeries[] results, TimeSeries[] correct) {
        //soso for efficiency, but data is small.
        int counter = 0;
        for (int i = 0; i < correct.length; i++) {
            for (int j = 0; j < results.length; j++) {
                if (correct[i] == results[j]) {
                    counter++;
                    break;
                }
            }
        }
        return counter;
    }

}
